Package: MuMIn
Type: Package
Title: Multi-Model Inference
Version: 1.48.19
Date: 2026-03-27
Encoding: UTF-8
Authors@R: c(
  person("Kamil", "Bartoń", role=c("aut", "cre"), 
         email="kamil.barton@go2.pl",
         comment = c(ORCID = "0000-0001-5562-8274"))
  )
Author: Kamil Bartoń [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-5562-8274>)
Maintainer: Kamil Bartoń <kamil.barton@go2.pl>
Description: Tools for model selection and model averaging with support for a 
             wide range of statistical models. Automated model selection through
             subsets of the maximum model, with optional constraints for model 
             inclusion. Averaging of model parameters and predictions based on 
             model weights derived from information criteria (AICc and alike) 
             or custom model weighting schemes.
License: GPL-2
Depends: R (>= 4.4.0)
Imports: graphics, methods, Matrix, stats, stats4, nlme, insight
Suggests: lme4 (>= 1.1.0), mgcv (>= 1.7.5), gamm4, MASS, nnet, survival
        (>= 3.1.0), geepack, performance
Enhances: aod, betareg, caper, coxme (>= 2.2.4), cplm, gee, glmmML,
        logistf, MCMCglmm, ordinal, pscl, spatialreg, splm, unmarked
        (>= 1.5.1), geeM (>= 0.7.5), gamlss, RMark, glmmTMB, brglm,
        quantreg, maxlike
LazyData: yes
NeedsCompilation: no
Packaged: 2026-03-27 09:09:51 UTC; kb2
Repository: CRAN
Date/Publication: 2026-03-27 19:50:02 UTC
Built: R 4.6.0; ; 2026-04-27 06:03:36 UTC; unix
